/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* cft_subscriber.c

   A subscription example

   This file is derived from code automatically generated by the rtiddsgen
   command:

   rtiddsgen -language C -example <arch> cft.idl

   Example subscription of type cft automatically generated by
   'rtiddsgen'. To test them, follow these steps:

   (1) Compile this file and the example publication.

   (2) Start the subscription with the command
       objs/<arch>/cft_subscriber <domain_id> <sample_count>

   (3) Start the publication with the command
       objs/<arch>/cft_publisher <domain_id> <sample_count>

   (4) [Optional] Specify the list of discovery initial peers and
       multicast receive addresses via an environment variable or a file
       (in the current working directory) called NDDS_DISCOVERY_PEERS.

   You can run any number of publishers and subscribers programs, and can
   add and remove them dynamically from the domain.


   Example:

       To run the example application on domain <domain_id>:

       On UNIX systems:

       objs/<arch>/cft_publisher <domain_id>
       objs/<arch>/cft_subscriber <domain_id>

       On Windows systems:

       objs\<arch>\cft_publisher <domain_id>
       objs\<arch>\cft_subscriber <domain_id>


modification history
------------ -------
*/

#include "cft.h"
#include "cftSupport.h"
#include "ndds/ndds_c.h"
#include <stdio.h>
#include <stdlib.h>

void cftListener_on_requested_deadline_missed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedDeadlineMissedStatus *status)
{
}

void cftListener_on_requested_incompatible_qos(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedIncompatibleQosStatus *status)
{
}

void cftListener_on_sample_rejected(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleRejectedStatus *status)
{
}

void cftListener_on_liveliness_changed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_LivelinessChangedStatus *status)
{
}

void cftListener_on_sample_lost(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleLostStatus *status)
{
}

void cftListener_on_subscription_matched(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SubscriptionMatchedStatus *status)
{
}

void cftListener_on_data_available(void *listener_data, DDS_DataReader *reader)
{
    cftDataReader *cft_reader = NULL;
    struct cftSeq data_seq = DDS_SEQUENCE_INITIALIZER;
    struct DDS_SampleInfoSeq info_seq = DDS_SEQUENCE_INITIALIZER;
    DDS_ReturnCode_t retcode;
    int i;

    cft_reader = cftDataReader_narrow(reader);
    if (cft_reader == NULL) {
        printf("DataReader narrow error\n");
        return;
    }

    retcode = cftDataReader_take(
            cft_reader,
            &data_seq,
            &info_seq,
            DDS_LENGTH_UNLIMITED,
            DDS_ANY_SAMPLE_STATE,
            DDS_ANY_VIEW_STATE,
            DDS_ANY_INSTANCE_STATE);
    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < cftSeq_get_length(&data_seq); ++i) {
        if (DDS_SampleInfoSeq_get_reference(&info_seq, i)->valid_data) {
            cftTypeSupport_print_data(cftSeq_get_reference(&data_seq, i));
        }
    }

    retcode = cftDataReader_return_loan(cft_reader, &data_seq, &info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }
}

/* Delete all entities */
static int subscriber_shutdown(DDS_DomainParticipant *participant)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides the finalize_instance() method on
       domain participant factory for users who want to release memory used
       by the participant factory. Uncomment the following block of code for
       clean destruction of the singleton. */
    /*
        retcode = DDS_DomainParticipantFactory_finalize_instance();
        if (retcode != DDS_RETCODE_OK) {
            printf("finalize_instance error %d\n", retcode);
            status = -1;
        }
    */

    return status;
}

static int subscriber_main(int domainId, int sample_count, int sel_cft)
{
    DDS_DomainParticipant *participant = NULL;
    DDS_Subscriber *subscriber = NULL;
    DDS_Topic *topic = NULL;
    struct DDS_DataReaderListener reader_listener =
            DDS_DataReaderListener_INITIALIZER;
    DDS_DataReader *reader = NULL;
    DDS_ReturnCode_t retcode;
    const char *type_name = NULL;
    int count = 0;
    struct DDS_Duration_t poll_period = { 1, 0 };
    DDS_ContentFilteredTopic *cft = NULL;
    /* If you want to modify datareader_qos programatically you
     * will need to use the following structure. */
    /* struct DDS_DataReaderQos datareader_qos = DDS_DataReaderQos_INITIALIZER;
     */
    /* For this filter we only allow 1 parameter*/
    struct DDS_StringSeq parameters;
    const char *param_list[] = { "SOME_STRING" };

    /* To customize participant QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domainId,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* To customize subscriber QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    subscriber = DDS_DomainParticipant_create_subscriber(
            participant,
            &DDS_SUBSCRIBER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (subscriber == NULL) {
        printf("create_subscriber error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* Register the type before creating the topic */
    type_name = cftTypeSupport_get_type_name();
    retcode = cftTypeSupport_register_type(participant, type_name);
    if (retcode != DDS_RETCODE_OK) {
        printf("register_type error %d\n", retcode);
        subscriber_shutdown(participant);
        return -1;
    }

    /* To customize topic QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    topic = DDS_DomainParticipant_create_topic(
            participant,
            "Example cft",
            type_name,
            &DDS_TOPIC_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (topic == NULL) {
        printf("create_topic error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    DDS_StringSeq_initialize(&parameters);
    DDS_StringSeq_set_maximum(&parameters, 1);
    /* Here we set the default filter using the param_list */
    DDS_StringSeq_from_array(&parameters, param_list, 1);

    if (sel_cft) {
        /* create_contentfilteredtopic_with_filter */
        cft = DDS_DomainParticipant_create_contentfilteredtopic_with_filter(
                participant,
                "ContentFilteredTopic",
                topic,
                "name MATCH %0",
                &parameters,
                DDS_STRINGMATCHFILTER_NAME);
        if (cft == NULL) {
            printf("create_contentfilteredtopic error\n");
            subscriber_shutdown(participant);
            return -1;
        }
    }

    /* Set up a data reader listener */
    reader_listener.on_requested_deadline_missed =
            cftListener_on_requested_deadline_missed;
    reader_listener.on_requested_incompatible_qos =
            cftListener_on_requested_incompatible_qos;
    reader_listener.on_sample_rejected = cftListener_on_sample_rejected;
    reader_listener.on_liveliness_changed = cftListener_on_liveliness_changed;
    reader_listener.on_sample_lost = cftListener_on_sample_lost;
    reader_listener.on_subscription_matched =
            cftListener_on_subscription_matched;
    reader_listener.on_data_available = cftListener_on_data_available;

    /* To customize data reader QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    if (sel_cft) {
        printf("Using ContentFiltered Topic\n");
        reader = DDS_Subscriber_create_datareader(
                subscriber,
                DDS_ContentFilteredTopic_as_topicdescription(cft),
                &DDS_DATAREADER_QOS_DEFAULT,
                &reader_listener,
                DDS_STATUS_MASK_ALL);
    } else {
        printf("Using Normal Topic\n");
        reader = DDS_Subscriber_create_datareader(
                subscriber,
                DDS_Topic_as_topicdescription(topic),
                &DDS_DATAREADER_QOS_DEFAULT,
                &reader_listener,
                DDS_STATUS_MASK_ALL);
    }
    if (reader == NULL) {
        printf("create_datareader error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* If you want to set the reliability and history QoS settings
     * programmatically rather than using the XML, you will need to add
     * the following lines to your code and comment out the create_datareader
     * calls above.
     */

    /*
    retcode = DDS_Subscriber_get_default_datareader_qos(subscriber,
        &datareader_qos);
    if (retcode != DDS_RETCODE_OK) {
        printf("get_default_datareader_qos error\n");
        return -1;
    }

    datareader_qos.reliability.kind = DDS_RELIABLE_RELIABILITY_QOS;
    datareader_qos.durability.kind = DDS_TRANSIENT_LOCAL_DURABILITY_QOS;
    datareader_qos.history.kind = DDS_KEEP_LAST_HISTORY_QOS;
    datareader_qos.history.depth = 20;

    if (sel_cft) {
        printf("Using ContentFiltered Topic\n");
        reader = DDS_Subscriber_create_datareader(
            subscriber, DDS_ContentFilteredTopic_as_topicdescription(cft),
            &datareader_qos, &reader_listener, DDS_STATUS_MASK_ALL);
    } else {
        printf("Using Normal Topic\n");
        reader = DDS_Subscriber_create_datareader(
            subscriber, DDS_Topic_as_topicdescription(topic),
            &datareader_qos, &reader_listener, DDS_STATUS_MASK_ALL);
    }
    if (reader == NULL) {
        printf("create_datareader error\n");
        subscriber_shutdown(participant);
        return -1;
    }
*/
    /* Change the filter */
    if (sel_cft) {
        printf(">>> Now setting a new filter: name MATCH \"EVEN\"\n");
        retcode = DDS_ContentFilteredTopic_append_to_expression_parameter(
                cft,
                0,
                "EVEN");
        if (retcode != DDS_RETCODE_OK) {
            printf("append_to_expression_parameter error\n");
            subscriber_shutdown(participant);
            return -1;
        }
    }

    /* Main loop */
    for (count = 0; (sample_count == 0) || (count < sample_count); ++count) {
        NDDS_Utility_sleep(&poll_period);

        if (sel_cft == 0) {
            continue;
        }
        if (count == 10) {
            printf("\n===========================\n");
            printf("Changing filter parameters\n");
            printf("Append 'ODD' filter\n");
            printf("===========================\n");
            retcode = DDS_ContentFilteredTopic_append_to_expression_parameter(
                    cft,
                    0,
                    "ODD");
            if (retcode != DDS_RETCODE_OK) {
                printf("append_to_expression_parameter error\n");
                subscriber_shutdown(participant);
                return -1;
            }
        } else if (count == 20) {
            printf("\n===========================\n");
            printf("Changing filter parameters\n");
            printf("Removing 'EVEN' filter \n");
            printf("===========================\n");

            retcode = DDS_ContentFilteredTopic_remove_from_expression_parameter(
                    cft,
                    0,
                    "EVEN");
            if (retcode != DDS_RETCODE_OK) {
                printf("append_to_expression_parameter error\n");
                subscriber_shutdown(participant);
                return -1;
            }
        }
    }

    /* Cleanup and delete all entities */
    return subscriber_shutdown(participant);
}

#if defined(RTI_WINCE)
int wmain(int argc, wchar_t **argv)
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = _wtoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = _wtoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#elif !(defined(RTI_VXWORKS) && !defined(__RTP__)) && !defined(RTI_PSOS)
int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */
    int sel_cft = 1;

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }
    if (argc >= 4) {
        sel_cft = atoi(argv[3]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count, sel_cft);
}
#endif

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "sub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) subscriber_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
