/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* deadline_contentfilter_subscriber.c

   A subscription example

   This file is derived from code automatically generated by the rtiddsgen
   command:

   rtiddsgen -language C -example <arch> deadline_contentfilter.idl

   Example subscription of type deadline_contentfilter automatically generated
   by 'rtiddsgen'. To test them, follow these steps:

   (1) Compile this file and the example publication.

   (2) Start the subscription with the command
       objs/<arch>/deadline_contentfilter_subscriber <domain_id> <sample_count>

   (3) Start the publication with the command
       objs/<arch>/deadline_contentfilter_publisher <domain_id> <sample_count>

   (4) [Optional] Specify the list of discovery initial peers and
       multicast receive addresses via an environment variable or a file
       (in the current working directory) called NDDS_DISCOVERY_PEERS.

   You can run any number of publishers and subscribers programs, and can
   add and remove them dynamically from the domain.


   Example:

       To run the example application on domain <domain_id>:

       On UNIX systems:

       objs/<arch>/deadline_contentfilter_publisher <domain_id>
       objs/<arch>/deadline_contentfilter_subscriber <domain_id>

       On Windows systems:

       objs\<arch>\deadline_contentfilter_publisher <domain_id>
       objs\<arch>\deadline_contentfilter_subscriber <domain_id>


modification history
------------ -------
* Create clock to show relative timing of events

* Define listener for requested deadline missed status

* Set deadline QoS

* Create content filter that ignores second instance
  after 10 seconds
*/

#include "deadline_contentfilter.h"
#include "deadline_contentfilterSupport.h"
#include "ndds/ndds_c.h"
#include <stdio.h>
#include <stdlib.h>

/* Start changes for Deadline */
/* For timekeeping */
#include <time.h>
clock_t init;

void deadline_contentfilterListener_on_requested_deadline_missed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedDeadlineMissedStatus *status)
{
    double elapsed_ticks;
    double elapsed_secs;
    deadline_contentfilter dummy;

    DDS_ReturnCode_t retcode = deadline_contentfilterDataReader_get_key_value(
            deadline_contentfilterDataReader_narrow(reader),
            &dummy,
            &status->last_instance_handle);
    if (retcode != DDS_RETCODE_OK) {
        printf("get_key_value error %d\n", retcode);
        return;
    }

    elapsed_ticks = clock() - init;
    elapsed_secs = elapsed_ticks / CLOCKS_PER_SEC;

    printf("Missed deadline @ t=%.2fs on instance code = %d\n",
           elapsed_secs,
           dummy.code);
}

/* End changes for Deadline */

void deadline_contentfilterListener_on_requested_incompatible_qos(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedIncompatibleQosStatus *status)
{
}

void deadline_contentfilterListener_on_sample_rejected(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleRejectedStatus *status)
{
}

void deadline_contentfilterListener_on_liveliness_changed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_LivelinessChangedStatus *status)
{
}

void deadline_contentfilterListener_on_sample_lost(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleLostStatus *status)
{
}

void deadline_contentfilterListener_on_subscription_matched(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SubscriptionMatchedStatus *status)
{
}

void deadline_contentfilterListener_on_data_available(
        void *listener_data,
        DDS_DataReader *reader)
{
    deadline_contentfilterDataReader *deadline_contentfilter_reader = NULL;
    struct deadline_contentfilterSeq data_seq = DDS_SEQUENCE_INITIALIZER;
    struct DDS_SampleInfoSeq info_seq = DDS_SEQUENCE_INITIALIZER;
    DDS_ReturnCode_t retcode;
    int i;
    double elapsed_ticks;
    double elapsed_secs;

    deadline_contentfilter_reader =
            deadline_contentfilterDataReader_narrow(reader);
    if (deadline_contentfilter_reader == NULL) {
        printf("DataReader narrow error\n");
        return;
    }

    retcode = deadline_contentfilterDataReader_take(
            deadline_contentfilter_reader,
            &data_seq,
            &info_seq,
            DDS_LENGTH_UNLIMITED,
            DDS_ANY_SAMPLE_STATE,
            DDS_ANY_VIEW_STATE,
            DDS_ANY_INSTANCE_STATE);
    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < deadline_contentfilterSeq_get_length(&data_seq); ++i) {
        if (DDS_SampleInfoSeq_get_reference(&info_seq, i)->valid_data) {
            deadline_contentfilter *data =
                    deadline_contentfilterSeq_get_reference(&data_seq, i);
            /* Start changes for Deadline */
            /* print the time we get each sample. */
            elapsed_ticks = clock() - init;
            elapsed_secs = elapsed_ticks / CLOCKS_PER_SEC;

            printf("@ t=%.2fs, Instance%d: <%d,%d>\n",
                   elapsed_secs,
                   data->code,
                   data->x,
                   data->y);
            /* End changes for Deadline */
        }
    }

    retcode = deadline_contentfilterDataReader_return_loan(
            deadline_contentfilter_reader,
            &data_seq,
            &info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }
}

/* Delete all entities */
static int subscriber_shutdown(DDS_DomainParticipant *participant)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides the finalize_instance() method on
       domain participant factory for users who want to release memory used
       by the participant factory. Uncomment the following block of code for
       clean destruction of the singleton. */
    /*
        retcode = DDS_DomainParticipantFactory_finalize_instance();
        if (retcode != DDS_RETCODE_OK) {
            printf("finalize_instance error %d\n", retcode);
            status = -1;
        }
    */

    return status;
}

static int subscriber_main(int domainId, int sample_count)
{
    DDS_DomainParticipant *participant = NULL;
    DDS_Subscriber *subscriber = NULL;
    DDS_Topic *topic = NULL;
    struct DDS_DataReaderListener reader_listener =
            DDS_DataReaderListener_INITIALIZER;
    DDS_DataReader *reader = NULL;
    DDS_ReturnCode_t retcode;
    const char *type_name = NULL;
    int count = 0;
    struct DDS_Duration_t poll_period = { 1, 0 };
    struct DDS_StringSeq parameters;
    const char *param_list[] = { "2" };
    DDS_ContentFilteredTopic *cft = NULL;
    /* struct DDS_DataReaderQos datareader_qos = DDS_DataReaderQos_INITIALIZER;
    struct DDS_Duration_t deadline_period = {2, 0}; */

    /* Changes for Deadline */
    /* for timekeeping */
    init = clock();

    /* To customize participant QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domainId,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* To customize subscriber QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    subscriber = DDS_DomainParticipant_create_subscriber(
            participant,
            &DDS_SUBSCRIBER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (subscriber == NULL) {
        printf("create_subscriber error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* Register the type before creating the topic */
    type_name = deadline_contentfilterTypeSupport_get_type_name();
    retcode = deadline_contentfilterTypeSupport_register_type(
            participant,
            type_name);
    if (retcode != DDS_RETCODE_OK) {
        printf("register_type error %d\n", retcode);
        subscriber_shutdown(participant);
        return -1;
    }

    /* To customize topic QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    topic = DDS_DomainParticipant_create_topic(
            participant,
            "Example deadline_contentfilter",
            type_name,
            &DDS_TOPIC_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (topic == NULL) {
        printf("create_topic error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* Set up a data reader listener */
    reader_listener.on_requested_deadline_missed =
            deadline_contentfilterListener_on_requested_deadline_missed;
    reader_listener.on_requested_incompatible_qos =
            deadline_contentfilterListener_on_requested_incompatible_qos;
    reader_listener.on_sample_rejected =
            deadline_contentfilterListener_on_sample_rejected;
    reader_listener.on_liveliness_changed =
            deadline_contentfilterListener_on_liveliness_changed;
    reader_listener.on_sample_lost =
            deadline_contentfilterListener_on_sample_lost;
    reader_listener.on_subscription_matched =
            deadline_contentfilterListener_on_subscription_matched;
    reader_listener.on_data_available =
            deadline_contentfilterListener_on_data_available;

    /* Start changes for Deadline */
    /* Set up content filtered topic to show interaction with deadline */

    DDS_StringSeq_initialize(&parameters);
    DDS_StringSeq_set_maximum(&parameters, 1);
    DDS_StringSeq_from_array(&parameters, param_list, 1);

    cft = DDS_DomainParticipant_create_contentfilteredtopic(
            participant,
            "ContentFilteredTopic",
            topic,
            "code < %0",
            &parameters);
    if (cft == NULL) {
        printf("create_contentfilteredtopic error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* To customize data reader QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    reader = DDS_Subscriber_create_datareader(
            subscriber,
            DDS_Topic_as_topicdescription(cft),
            &DDS_DATAREADER_QOS_DEFAULT,
            &reader_listener,
            DDS_STATUS_MASK_ALL);
    if (reader == NULL) {
        printf("create_datareader error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* If you want to change the DataReader's QoS programmatically rather than
     * using the XML file, you will need to add the following lines to your
     * code and comment out the create_datareader call above.
     *
     * In this case, we set the deadline period to 2 seconds to trigger
     * a deadline if the DataWriter does not update often enough, or if
     * the content-filter filters out data so there is no data available
     * with 2 seconds.
     */
    /*
    retcode = DDS_Subscriber_get_default_datareader_qos(subscriber,
                    &datareader_qos);
    if (retcode != DDS_RETCODE_OK) {
        printf("get_default_datareader_qos error\n");
        return -1;
    }

*/ /* Set deadline QoS */
    /*    datareader_qos.deadline.period = deadline_period;

        reader = DDS_Subscriber_create_datareader(
            subscriber, DDS_ContentFilteredTopic_as_topicdescription(cft),
            &datareader_qos, &reader_listener, DDS_STATUS_MASK_ALL);
        if (reader == NULL) {
            printf("create_datareader error\n");
            subscriber_shutdown(participant);
            return -1;
        }
        */

    /* Main loop */
    for (count = 0; (sample_count == 0) || (count < sample_count); ++count) {
        if (count == 10) {
            printf("Starting to filter out instance1 at time %d\n", count);
            DDS_String_free(DDS_StringSeq_get(&parameters, 0));
            *DDS_StringSeq_get_reference(&parameters, 0) = DDS_String_dup("1");

            DDS_ContentFilteredTopic_set_expression_parameters(
                    cft,
                    &parameters);
        }
        NDDS_Utility_sleep(&poll_period);
    }

    /* End changes for Deadline */

    /* Cleanup and delete all entities */
    return subscriber_shutdown(participant);
}

#if defined(RTI_WINCE)
int wmain(int argc, wchar_t **argv)
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = _wtoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = _wtoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#elif !(defined(RTI_VXWORKS) && !defined(__RTP__)) && !defined(RTI_PSOS)
int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#endif

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "sub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) subscriber_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
