/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* Shapes_subscriber.c

 A subscription of data of type ShapeType using DynamicData

 This file is derived from code automatically generated by the rtiddsgen
 command:

 rtiddsgen -language C -example <arch> Shapes.idl

 If you are not using and IDL to generate the TypeCode,
 you will need to create it manually. Follow other DynamicData
 examples (such as NestedStruct one) to learn how to do it.

 Example:

 To run the example application on domain <domain_id>:

 On UNIX systems:

 objs/<arch>/Shapes_publisher <domain_id> <sample #>
 objs/<arch>/Shapes_subscriber <domain_id> <sample #>

 On Windows systems:

 objs\<arch>\Shapes_publisher <domain_id> <sample #>
 objs\<arch>\Shapes_subscriber <domain_id> <sample #>


 */

#include "Shapes.h"
#include "ndds/ndds_c.h"
#include <stdio.h>
#include <stdlib.h>

#define EXAMPLE_TYPE_NAME "ShapesType"

void ShapeTypeListener_on_requested_deadline_missed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedDeadlineMissedStatus *status)
{
}

void ShapeTypeListener_on_requested_incompatible_qos(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedIncompatibleQosStatus *status)
{
}

void ShapeTypeListener_on_sample_rejected(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleRejectedStatus *status)
{
}

void ShapeTypeListener_on_liveliness_changed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_LivelinessChangedStatus *status)
{
}

void ShapeTypeListener_on_sample_lost(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleLostStatus *status)
{
}

void ShapeTypeListener_on_subscription_matched(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SubscriptionMatchedStatus *status)
{
}

void ShapeTypeListener_on_data_available(
        void *listener_data,
        DDS_DataReader *reader)
{
    /* We need to create a DynamicDataReader to receive the DynamicData
     * and a DynamicDataSeq to store there the available DynamicData received
     */
    DDS_DynamicDataReader *DynamicData_reader = NULL;
    struct DDS_DynamicDataSeq data_seq = DDS_SEQUENCE_INITIALIZER;
    struct DDS_SampleInfoSeq info_seq = DDS_SEQUENCE_INITIALIZER;
    DDS_ReturnCode_t retcode;
    int i;

    /* To use DynamicData, we need to assign the generic DataReader to
     * a DynamicDataReader, using DDS_DynamicDataReader_narrow.
     * The following narrow fuction should never fail, as it performs
     * only a safe cast.
     */
    DynamicData_reader = DDS_DynamicDataReader_narrow(reader);
    if (DynamicData_reader == NULL) {
        printf("DataReader narrow error\n");
        return;
    }

    retcode = DDS_DynamicDataReader_take(
            DynamicData_reader,
            &data_seq,
            &info_seq,
            DDS_LENGTH_UNLIMITED,
            DDS_ANY_SAMPLE_STATE,
            DDS_ANY_VIEW_STATE,
            DDS_ANY_INSTANCE_STATE);
    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < DDS_DynamicDataSeq_get_length(&data_seq); ++i) {
        if (DDS_SampleInfoSeq_get_reference(&info_seq, i)->valid_data) {
            DDS_DynamicData_print(
                    DDS_DynamicDataSeq_get_reference(&data_seq, i),
                    stdout,
                    1);
        }
    }

    retcode = DDS_DynamicDataReader_return_loan(
            DynamicData_reader,
            &data_seq,
            &info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }

    DDS_DynamicDataSeq_finalize(&data_seq);
    DDS_SampleInfoSeq_finalize(&info_seq);
}

/* Delete all entities */
static int subscriber_shutdown(
        DDS_DomainParticipant *participant,
        struct DDS_DynamicDataTypeSupport *typeSupport)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        if (typeSupport != NULL) {
            DDS_DynamicDataTypeSupport_delete(typeSupport);
        }

        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides the finalize_instance() method on
     * domain participant factory for users who want to release memory used
     * by the participant factory. Uncomment the following block of code for
     * clean destruction of the singleton. */

    retcode = DDS_DomainParticipantFactory_finalize_instance();
    if (retcode != DDS_RETCODE_OK) {
        printf("finalize_instance error %d\n", retcode);
        status = -1;
    }

    return status;
}

static int subscriber_main(int domainId, int sample_count)
{
    DDS_DomainParticipant *participant = NULL;
    DDS_Subscriber *subscriber = NULL;
    DDS_Topic *topic = NULL;
    struct DDS_DataReaderListener reader_listener =
            DDS_DataReaderListener_INITIALIZER;
    DDS_DataReader *reader = NULL;
    DDS_ReturnCode_t retcode;
    const char *type_name = NULL;
    int count = 0;
    struct DDS_Duration_t poll_period = { 1, 0 };

    /* Dynamic Data parameters that we will need */
    struct DDS_TypeCode *type_code = NULL;
    struct DDS_DynamicDataTypeProperty_t props =
            DDS_DynamicDataTypeProperty_t_INITIALIZER;
    struct DDS_DynamicDataTypeSupport *type_support = NULL;

    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domainId,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    subscriber = DDS_DomainParticipant_create_subscriber(
            participant,
            &DDS_SUBSCRIBER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (subscriber == NULL) {
        printf("create_subscriber error\n");
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    /* Create DynamicData using TypeCode from Shapes.c
     * If you are NOT using a type generated with rtiddsgen, you
     * need to create this TypeCode from scratch.
     */
    type_code = ShapeType_get_typecode();
    if (type_code == NULL) {
        printf("get_typecode error\n");
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    /* Create the Dynamic data type support object */
    type_support = DDS_DynamicDataTypeSupport_new(type_code, &props);
    if (type_support == NULL) {
        fprintf(stderr, "! Unable to create dynamic data type support\n");
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    /* Register the type before creating the topic */
    type_name = EXAMPLE_TYPE_NAME;
    retcode = DDS_DynamicDataTypeSupport_register_type(
            type_support,
            participant,
            type_name);
    if (retcode != DDS_RETCODE_OK) {
        printf("register_type error %d\n", retcode);
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    /* Make sure both publisher and subscriber share the same topic name.
     * In the Shapes example: we are subscribing to a Square, wich is the
     * topic name. If you want to publish other shapes (Triangle or Circle),
     * you just need to update the topic name.
     */
    topic = DDS_DomainParticipant_create_topic(
            participant,
            "Square",
            type_name,
            &DDS_TOPIC_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (topic == NULL) {
        printf("create_topic error\n");
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    /* Set up a data reader listener */
    reader_listener.on_requested_deadline_missed =
            ShapeTypeListener_on_requested_deadline_missed;
    reader_listener.on_requested_incompatible_qos =
            ShapeTypeListener_on_requested_incompatible_qos;
    reader_listener.on_sample_rejected = ShapeTypeListener_on_sample_rejected;
    reader_listener.on_liveliness_changed =
            ShapeTypeListener_on_liveliness_changed;
    reader_listener.on_sample_lost = ShapeTypeListener_on_sample_lost;
    reader_listener.on_subscription_matched =
            ShapeTypeListener_on_subscription_matched;
    reader_listener.on_data_available = ShapeTypeListener_on_data_available;

    /* First, we create a generic DataReader for our topic */
    reader = DDS_Subscriber_create_datareader(
            subscriber,
            DDS_Topic_as_topicdescription(topic),
            &DDS_DATAREADER_QOS_DEFAULT,
            &reader_listener,
            DDS_STATUS_MASK_ALL);
    if (reader == NULL) {
        printf("create_datareader error\n");
        subscriber_shutdown(participant, type_support);
        return -1;
    }

    /* Main loop */
    for (count = 0; (sample_count == 0) || (count < sample_count); ++count) {
        printf("ShapeType subscriber sleeping for %d sec...\n",
               poll_period.sec);
        NDDS_Utility_sleep(&poll_period);
    }

    /* Cleanup and delete all entities */
    return subscriber_shutdown(participant, type_support);
}

#if defined(RTI_WINCE)
int wmain(int argc, wchar_t **argv)
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = _wtoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = _wtoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
     NDDS_Config_Logger_set_verbosity_by_category(
     NDDS_Config_Logger_get_instance(),
     NDDS_CONFIG_LOG_CATEGORY_API,
     NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
     */

    return subscriber_main(domainId, sample_count);
}
#elif !(defined(RTI_VXWORKS) && !defined(__RTP__)) && !defined(RTI_PSOS)
int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
     NDDS_Config_Logger_set_verbosity_by_category(
     NDDS_Config_Logger_get_instance(),
     NDDS_CONFIG_LOG_CATEGORY_API,
     NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
     */

    return subscriber_main(domainId, sample_count);
}
#endif

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "sub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) subscriber_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
