/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* Foo_publisher.c

 A publication of data of type Foo

 This file is derived from code automatically generated by the rtiddsgen
 command:

 rtiddsgen -language C -example <arch> Foo.idl

 This example shows how to create some publshers and how to access all
 the publishers the participant has.

 Example:

 To run the example application on domain <domain_id>:

 On Unix:

 objs/<arch>/Foo_publisher <domain_id>

 On Windows:

 objs\<arch>\Foo_publisher <domain_id>


 modification history
 ------------ -------
 */

#include "Foo.h"
#include "FooSupport.h"
#include "ndds/ndds_c.h"
#include <stdio.h>
#include <stdlib.h>

/* Delete all entities */
static int publisher_shutdown(
        DDS_DomainParticipant *participant,
        struct DDS_PublisherSeq publisherSeq)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }

        if (DDS_PublisherSeq_finalize(&publisherSeq)) {
            printf("finalize publisherSeq error\n");
            status = -1;
        }
    }

    /* RTI Connext provides finalize_instance() method on
     * domain participant factory for people who want to release memory used
     * by the participant factory. Uncomment the following block of code for
     * clean destruction of the singleton. */
    /*
     retcode = DDS_DomainParticipantFactory_finalize_instance();
     if (retcode != DDS_RETCODE_OK) {
     printf("finalize_instance error %d\n", retcode);
     status = -1;
     }
     */

    return status;
}

static int publisher_main(int domainId, int sample_count)
{
    DDS_DomainParticipant *participant = NULL;
    DDS_Publisher *publisher = NULL, *publisher2 = NULL;
    /* We will use this temporal publisher to print all created publishers */
    DDS_Publisher *tmp = NULL;
    struct DDS_PublisherSeq publisherSeq = DDS_SEQUENCE_INITIALIZER;
    Foo *instance = NULL;
    DDS_ReturnCode_t retcode;
    const char *type_name = NULL;
    int count = 0;

    /* To customize participant QoS, use
     * the configuration file USER_QOS_PROFILES.xml */
    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domainId,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        publisher_shutdown(participant, publisherSeq);
        return -1;
    }

    /* Start - modifying the generated example to showcase the usage of the
     * get_publishers API
     */

    /* To customize publisher QoS, use
     the configuration file USER_QOS_PROFILES.xml */
    publisher = DDS_DomainParticipant_create_publisher(
            participant,
            &DDS_PUBLISHER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (publisher == NULL) {
        printf("create_publisher error\n");
        publisher_shutdown(participant, publisherSeq);
        return -1;
    }

    printf("The first publisher is %p\n", publisher);

    publisher2 = DDS_DomainParticipant_create_publisher(
            participant,
            &DDS_PUBLISHER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (publisher2 == NULL) {
        printf("create_publisher2 error\n");
        publisher_shutdown(participant, publisherSeq);
        return -1;
    }

    printf("The second publisher is %p\n", publisher2);
    printf("Let's call get_publisher() now and check if I get all my ");
    printf("publisher...\n");

    retcode = DDS_DomainParticipant_get_publishers(participant, &publisherSeq);
    if (retcode != DDS_RETCODE_OK) {
        printf("get_publishers error\n");
        publisher_shutdown(participant, publisherSeq);
        return -1;
    }

    /* We print the publishers obtained */
    for (count = 0; count < DDS_PublisherSeq_get_length(&publisherSeq);
         ++count) {
        tmp = DDS_PublisherSeq_get(&publisherSeq, count);
        printf("The %d publisher I foud is: %p\n", count, tmp);
    }

    /* Cleanup and delete delete all entities */
    return publisher_shutdown(participant, publisherSeq);
    /* End - modifying the generated example to showcase the usage of the
     * get_publishers API
     */
}

#if defined(RTI_WINCE)
int wmain(int argc, wchar_t **argv)
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = _wtoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = _wtoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
     NDDS_Config_Logger_set_verbosity_by_category(
     NDDS_Config_Logger_get_instance(),
     NDDS_CONFIG_LOG_CATEGORY_API,
     NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
     */

    return publisher_main(domainId, sample_count);
}
#elif !(defined(RTI_VXWORKS) && !defined(__RTP__)) && !defined(RTI_PSOS)
int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
     NDDS_Config_Logger_set_verbosity_by_category(
     NDDS_Config_Logger_get_instance(),
     NDDS_CONFIG_LOG_CATEGORY_API,
     NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
     */

    return publisher_main(domainId, sample_count);
}
#endif

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "pub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) publisher_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
