/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* profiles_subscriber.c

   A subscription example

   This file is derived from code automatically generated by the rtiddsgen
   command:

   rtiddsgen -language C -example <arch> profiles.idl

   Example subscription of type profiles automatically generated by
   'rtiddsgen'. To test them, follow these steps:

   (1) Compile this file and the example publication.

   (2) Start the subscription with the command
       objs/<arch>/profiles_subscriber <domain_id> <sample_count>

   (3) Start the publication with the command
       objs/<arch>/profiles_publisher <domain_id> <sample_count>

   (4) [Optional] Specify the list of discovery initial peers and
       multicast receive addresses via an environment variable or a file
       (in the current working directory) called NDDS_DISCOVERY_PEERS.

   You can run any number of publishers and subscribers programs, and can
   add and remove them dynamically from the domain.


   Example:

       To run the example application on domain <domain_id>:

       On UNIX systems:

       objs/<arch>/profiles_publisher <domain_id>
       objs/<arch>/profiles_subscriber <domain_id>

       On Windows systems:

       objs\<arch>\profiles_publisher <domain_id>
       objs\<arch>\profiles_subscriber <domain_id>


modification history
------------ -------
*/

#include "ndds/ndds_c.h"
#include "profiles.h"
#include "profilesSupport.h"
#include <stdio.h>
#include <stdlib.h>

#define NUM_PROFILE_FILES 1

void profilesListener_on_requested_deadline_missed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedDeadlineMissedStatus *status)
{
}

void profilesListener_on_requested_incompatible_qos(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_RequestedIncompatibleQosStatus *status)
{
}

void profilesListener_on_sample_rejected(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleRejectedStatus *status)
{
}

void profilesListener_on_liveliness_changed(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_LivelinessChangedStatus *status)
{
}

void profilesListener_on_sample_lost(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SampleLostStatus *status)
{
}

void profilesListener_on_subscription_matched(
        void *listener_data,
        DDS_DataReader *reader,
        const struct DDS_SubscriptionMatchedStatus *status)
{
}

void profilesListener_on_data_available(
        void *listener_data,
        DDS_DataReader *reader)
{
    profilesDataReader *profiles_reader = NULL;
    struct profilesSeq data_seq = DDS_SEQUENCE_INITIALIZER;
    struct DDS_SampleInfoSeq info_seq = DDS_SEQUENCE_INITIALIZER;
    DDS_ReturnCode_t retcode;
    int i;

    profiles_reader = profilesDataReader_narrow(reader);
    if (profiles_reader == NULL) {
        printf("DataReader narrow error\n");
        return;
    }

    retcode = profilesDataReader_take(
            profiles_reader,
            &data_seq,
            &info_seq,
            DDS_LENGTH_UNLIMITED,
            DDS_ANY_SAMPLE_STATE,
            DDS_ANY_VIEW_STATE,
            DDS_ANY_INSTANCE_STATE);
    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    printf("=============================================\n");
    printf("listener received\n");
    printf("=============================================\n");
    for (i = 0; i < profilesSeq_get_length(&data_seq); ++i) {
        if (DDS_SampleInfoSeq_get_reference(&info_seq, i)->valid_data) {
            profilesTypeSupport_print_data(
                    profilesSeq_get_reference(&data_seq, i));
        }
    }

    retcode = profilesDataReader_return_loan(
            profiles_reader,
            &data_seq,
            &info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }
}

/* Delete all entities */
static int subscriber_shutdown(
        DDS_DomainParticipant *participant,
        struct DDS_DomainParticipantFactoryQos *factory_qos)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    retcode = DDS_DomainParticipantFactoryQos_finalize(factory_qos);
    if (retcode != DDS_RETCODE_OK) {
        printf("FactoryQos_finalize error %d\n", retcode);
        status = -1;
    }

    if (participant != NULL) {
        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides the finalize_instance() method on
       domain participant factory for users who want to release memory used
       by the participant factory. Uncomment the following block of code for
       clean destruction of the singleton. */
    /*
        retcode = DDS_DomainParticipantFactory_finalize_instance();
        if (retcode != DDS_RETCODE_OK) {
            printf("finalize_instance error %d\n", retcode);
            status = -1;
        }
    */

    return status;
}

static int subscriber_main(int domainId, int sample_count)
{
    DDS_DomainParticipant *participant = NULL;
    DDS_Subscriber *subscriber = NULL;
    DDS_Topic *topic = NULL;
    struct DDS_DataReaderListener reader_volatile_listener =
            DDS_DataReaderListener_INITIALIZER;
    struct DDS_DataReaderListener reader_transient_local_listener =
            DDS_DataReaderListener_INITIALIZER;
    DDS_DataReader *reader_volatile = NULL;
    DDS_DataReader *reader_transient_local = NULL;
    DDS_ReturnCode_t retcode;
    const char *type_name = NULL;
    int count = 0;
    struct DDS_Duration_t poll_period = { 1, 0 };
    int status = 0;

    struct DDS_DomainParticipantFactoryQos factory_qos =
            DDS_DomainParticipantFactoryQos_INITIALIZER;
    static const char *myUrlProfile[NUM_PROFILE_FILES] = {
        "file://./my_custom_qos_profiles.xml"
    };

    /* There are several different approaches for loading QoS profiles from XML
     * files (see Configuring QoS with XML chapter in the RTI Connext Core
     * Libraries and Utilities User's Manual). In this example we illustrate
     * two of them:
     *
     * 1) Creating a file named USER_QOS_PROFILES.xml, which is loaded,
     * automatically by the DomainParticipantFactory. In this case, the file
     * defines a QoS profile named volatile_profile that configures reliable,
     * volatile DataWriters and DataReaders.
     *
     * 2) Adding XML documents to the DomainParticipantFactory using its
     * Profile QoSPolicy (DDS Extension). In this case, we add
     * my_custom_qos_profiles.xml to the url_profile sequence, which stores
     * the URLs of all the XML documents with QoS policies that are loaded by
     * the DomainParticipantFactory aside from the ones that are automatically
     * loaded.
     * my_custom_qos_profiles.xml defines a QoS profile named
     * transient_local_profile that configures reliable, transient local
     * DataWriters and DataReaders.
     */

    /* To load my_custom_qos_profiles.xml, as explained above, we need to modify
     * the  DDSTheParticipantFactory Profile QoSPolicy */
    DDS_DomainParticipantFactory_get_qos(
            DDS_TheParticipantFactory,
            &factory_qos);

    /* We are only going to add one XML file to the url_profile sequence, so our
     * NUM_PROFILE_FILES is 1 (defined at start) */
    DDS_StringSeq_from_array(
            &(factory_qos.profile.url_profile),
            (const char **) myUrlProfile,
            NUM_PROFILE_FILES);

    /* The XML file will be loaded from the working directory. That means, you
     * need to run the example like this:
     * ./objs/<architecture>/profiles_publisher
     * (see README.txt for more information on how to run the example).
     *
     * Note that you can specify the absolute path of the XML QoS file to avoid
     * this problem.
     */
    retcode = DDS_DomainParticipantFactory_set_qos(
            DDS_TheParticipantFactory,
            &factory_qos);
    if (retcode != DDS_RETCODE_OK) {
        printf("set_qos error %d\n", retcode);
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* Our default Qos profile, volatile_profile, sets the participant name.
     * This is the only participant_qos policy that we change in our
     * example. As this is done in the default QoS profile, we don't need
     * to specify its name, so we can create the participant using the
     * create_participant() method rather than using
     * create_participant_with_profile().  */
    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domainId,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* We haven't changed the subscriber_qos in any of QoS profiles we use in
     * this example, so we can just use the create_topic() method. If you want
     * to load an specific profile in which you may have changed the
     * publisher_qos, use the create_publisher_with_profile() method. */
    subscriber = DDS_DomainParticipant_create_subscriber(
            participant,
            &DDS_SUBSCRIBER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (subscriber == NULL) {
        printf("create_subscriber error\n");
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* Register the type before creating the topic */
    type_name = profilesTypeSupport_get_type_name();
    retcode = profilesTypeSupport_register_type(participant, type_name);
    if (retcode != DDS_RETCODE_OK) {
        printf("register_type error %d\n", retcode);
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* We haven't changed the topic_qos in any of QoS profiles we use in this
     * example, so we can just use the create_topic() method. If you want to
     * load an specific profile in which you may have changed the topic_qos,
     * use the create_topic_with_profile() method. */
    topic = DDS_DomainParticipant_create_topic(
            participant,
            "Example profiles",
            type_name,
            &DDS_TOPIC_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (topic == NULL) {
        printf("create_topic error\n");
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* Set up a data reader listener */
    reader_volatile_listener.on_requested_deadline_missed =
            profilesListener_on_requested_deadline_missed;
    reader_volatile_listener.on_requested_incompatible_qos =
            profilesListener_on_requested_incompatible_qos;
    reader_volatile_listener.on_sample_rejected =
            profilesListener_on_sample_rejected;
    reader_volatile_listener.on_liveliness_changed =
            profilesListener_on_liveliness_changed;
    reader_volatile_listener.on_sample_lost = profilesListener_on_sample_lost;
    reader_volatile_listener.on_subscription_matched =
            profilesListener_on_subscription_matched;
    reader_volatile_listener.on_data_available =
            profilesListener_on_data_available;

    reader_transient_local_listener.on_requested_deadline_missed =
            profilesListener_on_requested_deadline_missed;
    reader_transient_local_listener.on_requested_incompatible_qos =
            profilesListener_on_requested_incompatible_qos;
    reader_transient_local_listener.on_sample_rejected =
            profilesListener_on_sample_rejected;
    reader_transient_local_listener.on_liveliness_changed =
            profilesListener_on_liveliness_changed;
    reader_transient_local_listener.on_sample_lost =
            profilesListener_on_sample_lost;
    reader_transient_local_listener.on_subscription_matched =
            profilesListener_on_subscription_matched;
    reader_transient_local_listener.on_data_available =
            profilesListener_on_data_available;

    /* Volatile reader -- As volatile_profile is the default qos profile
     * we don't need to specify the profile we are going to use, we can
     * just call create_datareader passing DDS_DATAWRITER_QOS_DEFAULT. */
    reader_volatile = DDS_Subscriber_create_datareader(
            subscriber,
            DDS_Topic_as_topicdescription(topic),
            &DDS_DATAREADER_QOS_DEFAULT,
            &reader_volatile_listener,
            DDS_STATUS_MASK_ALL);
    if (reader_volatile == NULL) {
        printf("create_datareader error\n");
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* Transient Local writer -- In this case we use
     * create_datareader_with_profile, because we have to use a profile other
     * than the default one. This profile has been defined in
     * my_custom_qos_profiles.xml, but since we already loaded the XML file
     * we don't need to specify anything else. */
    reader_transient_local = DDS_Subscriber_create_datareader_with_profile(
            subscriber,
            DDS_Topic_as_topicdescription(topic),
            "profiles_Library",               /* library_name */
            "transient_local_profile",        /* profile_name */
            &reader_transient_local_listener, /* listener */
            DDS_STATUS_MASK_ALL);
    if (reader_transient_local == NULL) {
        printf("create_datareader error\n");
        subscriber_shutdown(participant, &factory_qos);
        return -1;
    }

    /* Main loop */
    for (count = 0; (sample_count == 0) || (count < sample_count); ++count) {
        printf("profiles subscriber sleeping for %d sec...\n", poll_period.sec);
        NDDS_Utility_sleep(&poll_period);
    }

    /* Cleanup and delete all entities */
    return subscriber_shutdown(participant, &factory_qos);
}

#if defined(RTI_WINCE)
int wmain(int argc, wchar_t **argv)
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = _wtoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = _wtoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#elif !(defined(RTI_VXWORKS) && !defined(__RTP__)) && !defined(RTI_PSOS)
int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#endif

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "sub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) subscriber_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
