/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* msg_subscriber.c

   A subscription example

   This file is derived from code automatically generated by the rtiddsgen
   command:

   rtiddsgen -language C -example <arch> msg.idl

   Example subscription of type msg automatically generated by
   'rtiddsgen'. To test them, follow these steps:

   (1) Compile this file and the example publication.

   (2) Start the subscription with the command
       objs/<arch>/msg_subscriber <domain_id> <sample_count>

   (3) Start the publication with the command
       objs/<arch>/msg_publisher <domain_id> <sample_count>

   (4) [Optional] Specify the list of discovery initial peers and
       multicast receive addresses via an environment variable or a file
       (in the current working directory) called NDDS_DISCOVERY_PEERS.

   You can run any number of publishers and subscribers programs, and can
   add and remove them dynamically from the domain.


   Example:

       To run the example application on domain <domain_id>:

       On UNIX systems:

       objs/<arch>/msg_publisher <domain_id>
       objs/<arch>/msg_subscriber <domain_id>

       On Windows systems:

       objs\<arch>\msg_publisher <domain_id>
       objs\<arch>\msg_subscriber <domain_id>


modification history
------------ -------
*/

#include "ndds/ndds_c.h"
#include <stdio.h>
#include <stdlib.h>

/* We use the typecode from built-in listeners, so we don't need to include
 * this code*/
/*
#include "msg.h"
#include "msgSupport.h"
*/


/* We are going to use the BuiltinPublicationListener_on_data_available
 * to detect the topics that are being published on the domain
 *
 * Once we have detected a new topic, we will print out the Topic Name,
 * Participant ID, DataWriter id, and Data Type.
 */
void BuiltinPublicationListener_on_data_available(
        void *listener_data,
        DDS_DataReader *reader)
{
    DDS_PublicationBuiltinTopicDataDataReader *builtin_reader = NULL;
    struct DDS_PublicationBuiltinTopicDataSeq data_seq =
            DDS_SEQUENCE_INITIALIZER;
    struct DDS_PublicationBuiltinTopicData *data = NULL;
    struct DDS_SampleInfoSeq info_seq = DDS_SEQUENCE_INITIALIZER;
    DDS_ReturnCode_t retcode;
    DDS_ExceptionCode_t exception_code;
    int i, len;

    builtin_reader = DDS_PublicationBuiltinTopicDataDataReader_narrow(reader);

    retcode = DDS_PublicationBuiltinTopicDataDataReader_take(
            builtin_reader,
            &data_seq,
            &info_seq,
            DDS_LENGTH_UNLIMITED,
            DDS_ANY_SAMPLE_STATE,
            DDS_NEW_VIEW_STATE,
            DDS_ANY_INSTANCE_STATE);

    if (retcode == DDS_RETCODE_NO_DATA)
        return;

    if (retcode != DDS_RETCODE_OK) {
        printf("***Error: failed to access data from the built-in reader\n");
        return;
    }

    len = DDS_PublicationBuiltinTopicDataSeq_get_length(&data_seq);
    for (i = 0; i < len; ++i) {
        if (DDS_SampleInfoSeq_get_reference(&info_seq, i)->valid_data) {
            data = DDS_PublicationBuiltinTopicDataSeq_get_reference(
                    &data_seq,
                    i);
            printf("-----\nFound topic \"%s\"\nparticipant: "
                   "%08x%08x%08x\ndatawriter: %08x%08x%08x\ntype:\n",
                   data->topic_name,
                   data->participant_key.value[0],
                   data->participant_key.value[1],
                   data->participant_key.value[2],
                   data->key.value[0],
                   data->key.value[1],
                   data->key.value[2]);

            if (data->type_code == NULL) {
                printf("No type code received, perhaps increase "
                       "type_code_max_serialized_length?\n");
                continue;
            }

            /* Using the type_code propagated we print the data type
             * with print_IDL(). */
            DDS_TypeCode_print_IDL(data->type_code, 2, &exception_code);
            if (exception_code != DDS_NO_EXCEPTION_CODE) {
                printf("Error***: print_IDL returns exception code %d",
                       exception_code);
            }
        }
    }
    DDS_PublicationBuiltinTopicDataDataReader_return_loan(
            builtin_reader,
            &data_seq,
            &info_seq);
}


/* Delete all entities */
static int subscriber_shutdown(DDS_DomainParticipant *participant)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides the finalize_instance() method on
       domain participant factory for users who want to release memory used
       by the participant factory. Uncomment the following block of code for
       clean destruction of the singleton. */
    /*
        retcode = DDS_DomainParticipantFactory_finalize_instance();
        if (retcode != DDS_RETCODE_OK) {
            printf("finalize_instance error %d\n", retcode);
            status = -1;
        }
    */

    return status;
}

static int subscriber_main(int domainId, int sample_count)
{
    DDS_DomainParticipant *participant = NULL;
    int count = 0;
    struct DDS_Duration_t poll_period = { 4, 0 };
    DDS_ReturnCode_t retcode;

    DDS_Subscriber *builtin_subscriber = NULL;
    DDS_PublicationBuiltinTopicDataDataReader *builtin_publication_datareader =
            NULL;

    /* Listener for the Built-in Publication Data Reader */
    struct DDS_DataReaderListener builtin_publication_listener =
            DDS_DataReaderListener_INITIALIZER;

    /* If you want to change the type_code_max_serialized_length
     * programmatically (e.g., to 3070) rather than using the XML file, you
     * will need to add the following lines to your code and comment out the
     * create_participant call bellow. */

    /*
    struct DDS_DomainParticipantQos participant_qos =
            DDS_DomainParticipantQos_INITIALIZER;
    retcode = DDS_DomainParticipantFactory_get_default_participant_qos(
            DDS_TheParticipantFactory, &participant_qos);
    if (retcode != DDS_RETCODE_OK) {
        printf("get_default_participant_qos error\n");
        return -1;
    }

    participant_qos.resource_limits.type_code_max_serialized_length = 3070;

    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory, domainId, &participant_qos,
            NULL, DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        subscriber_shutdown(participant);
        return -1;
    }
    */

    /* To customize participant QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domainId,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        subscriber_shutdown(participant);
        return -1;
    }

    /* First get the built-in subscriber */
    builtin_subscriber =
            DDS_DomainParticipant_get_builtin_subscriber(participant);
    if (builtin_subscriber == NULL) {
        printf("***Error: failed to create builtin subscriber\n");
        return 0;
    }

    /* Then get the data reader for the built-in subscriber */
    builtin_publication_datareader =
            (DDS_PublicationBuiltinTopicDataDataReader *)
                    DDS_Subscriber_lookup_datareader(
                            builtin_subscriber,
                            DDS_PUBLICATION_TOPIC_NAME);
    if (builtin_publication_datareader == NULL) {
        printf("***Error: failed to create builtin publication data reader\n");
        return 0;
    }

    /* Finally install the listener */
    builtin_publication_listener.on_data_available =
            BuiltinPublicationListener_on_data_available;
    DDS_DataReader_set_listener(
            (DDS_DataReader *) builtin_publication_datareader,
            &builtin_publication_listener,
            DDS_DATA_AVAILABLE_STATUS);


    /* Main loop */
    for (count = 0; (sample_count == 0) || (count < sample_count); ++count) {
        NDDS_Utility_sleep(&poll_period);
    }

    /* Cleanup and delete all entities */
    return subscriber_shutdown(participant);
}

#if defined(RTI_WINCE)
int wmain(int argc, wchar_t **argv)
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = _wtoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = _wtoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#elif !(defined(RTI_VXWORKS) && !defined(__RTP__)) && !defined(RTI_PSOS)
int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return subscriber_main(domainId, sample_count);
}
#endif

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "sub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) subscriber_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
