/*******************************************************************************
 (c) 2005-2014 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/
/* hello_world_publisher.c

   A publication of data of type hello_world

   This file is derived from code automatically generated by the rtiddsgen
   command:

   rtiddsgen -language C -example <arch> hello_world.idl

   Example publication of type hello_world automatically generated by
   'rtiddsgen'. To test them follow these steps:

   (1) Compile this file and the example subscription.

   (2) Start the subscription with the command
       objs/<arch>/hello_world_subscriber <domain_id> <sample_count>

   (3) Start the publication with the command
       objs/<arch>/hello_world_publisher <domain_id> <sample_count>

   (4) [Optional] Specify the list of discovery initial peers and
       multicast receive addresses via an environment variable or a file
       (in the current working directory) called NDDS_DISCOVERY_PEERS.

   You can run any number of publishers and subscribers programs, and can
   add and remove them dynamically from the domain.


   Example:

       To run the example application on domain <domain_id>:

       On Unix:

       objs/<arch>/hello_world_publisher <domain_id>
       objs/<arch>/hello_world_subscriber <domain_id>

       On Windows:

       objs\<arch>\hello_world_publisher <domain_id>
       objs\<arch>\hello_world_subscriber <domain_id>

*/

#include "hello_world.h"
#include "hello_worldSupport.h"
#include "ndds/ndds_c.h"
#include <stdio.h>
#include <stdlib.h>

/******************************************************************************
  Macros to read input parameters
******************************************************************************/

#define READ_INTEGER_PARAM(parameter, outValue) \
    if (!strcmp(argv[i], parameter)) {          \
        if (i + 1 >= argc) {                    \
            printf("%s", syntax);               \
            return 1;                           \
        }                                       \
        i++;                                    \
        outValue = atoi(argv[i]);               \
        i++;                                    \
        continue;                               \
    }

#define READ_STRING_PARAM(parameter, outValue) \
    if (!strcmp(argv[i], parameter)) {         \
        if (i + 1 >= argc) {                   \
            printf("%s", syntax);              \
            return 1;                          \
        }                                      \
        i++;                                   \
        outValue = rgv[i];                     \
        i++;                                   \
        continue;                              \
    }


/* Delete all entities */
static int publisher_shutdown(DDS_DomainParticipant *participant)
{
    DDS_ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        retcode = DDS_DomainParticipant_delete_contained_entities(participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = DDS_DomainParticipantFactory_delete_participant(
                DDS_TheParticipantFactory,
                participant);
        if (retcode != DDS_RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides finalize_instance() method on
       domain participant factory for people who want to release memory used
       by the participant factory. Uncomment the following block of code for
       clean destruction of the singleton. */
    /*
        retcode = DDS_DomainParticipantFactory_finalize_instance();
        if (retcode != DDS_RETCODE_OK) {
            printf("finalize_instance error %d\n", retcode);
            status = -1;
        }
    */

    return status;
}

static int publisher_main(
        int domain_id,
        int sample_count,
        int initial_count,
        int dwh,
        int sleep)
{
    DDS_DomainParticipant *participant = NULL;
    DDS_Publisher *publisher = NULL;
    DDS_Topic *topic = NULL;
    DDS_DataWriter *writer = NULL;
    struct DDS_DataWriterQos writer_qos = DDS_DataWriterQos_INITIALIZER;
    hello_worldDataWriter *hello_world_writer = NULL;
    hello_world *instance = NULL;
    DDS_ReturnCode_t retcode;
    DDS_InstanceHandle_t instance_handle = DDS_HANDLE_NIL;
    const char *type_name = NULL;
    int count = 0;
    struct DDS_Duration_t send_period = { 1, 0 };
    struct DDS_Duration_t one_sec = { 1, 0 };

    /* To customize participant QoS, use  the configuration file
     * USER_QOS_PROFILES.xml */
    participant = DDS_DomainParticipantFactory_create_participant(
            DDS_TheParticipantFactory,
            domain_id,
            &DDS_PARTICIPANT_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (participant == NULL) {
        printf("create_participant error\n");
        publisher_shutdown(participant);
        return -1;
    }

    /* To customize publisher QoS, use the configuration file
     * USER_QOS_PROFILES.xml */
    publisher = DDS_DomainParticipant_create_publisher(
            participant,
            &DDS_PUBLISHER_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (publisher == NULL) {
        printf("create_publisher error\n");
        publisher_shutdown(participant);
        return -1;
    }

    /* Register type before creating topic */
    type_name = hello_worldTypeSupport_get_type_name();
    retcode = hello_worldTypeSupport_register_type(participant, type_name);
    if (retcode != DDS_RETCODE_OK) {
        printf("register_type error %d\n", retcode);
        publisher_shutdown(participant);
        return -1;
    }

    /* To customize topic QoS, use
     the configuration file USER_QOS_PROFILES.xml */
    topic = DDS_DomainParticipant_create_topic(
            participant,
            "Example hello_world",
            type_name,
            &DDS_TOPIC_QOS_DEFAULT,
            NULL /* listener */,
            DDS_STATUS_MASK_NONE);
    if (topic == NULL) {
        printf("create_topic error\n");
        publisher_shutdown(participant);
        return -1;
    }

    /* If you use Durable Writer History, you need to set
     * several properties. These properties are set in the USER_QOS_PROFILE.xml
     * file, "durable_writer_history_Profile" profile. See that file for further
     * details.
     */
    if (dwh == 1) {
        writer = DDS_Publisher_create_datawriter_with_profile(
                publisher,
                topic,
                "persistence_example_Library",
                "durable_writer_history_Profile",
                NULL /* listener */,
                DDS_STATUS_MASK_NONE);
        if (writer == NULL) {
            printf("create_datawriter error\n");
            publisher_shutdown(participant);
            return -1;
        }
    } else {
        writer = DDS_Publisher_create_datawriter_with_profile(
                publisher,
                topic,
                "persistence_example_Library",
                "persistence_service_Profile",
                NULL /* listener */,
                DDS_STATUS_MASK_NONE);
        if (writer == NULL) {
            printf("create_datawriter error\n");
            publisher_shutdown(participant);
            return -1;
        }
    }

    hello_world_writer = hello_worldDataWriter_narrow(writer);
    if (hello_world_writer == NULL) {
        printf("DataWriter narrow error\n");
        publisher_shutdown(participant);
        return -1;
    }

    /* Create data sample for writing */
    instance = hello_worldTypeSupport_create_data_ex(DDS_BOOLEAN_TRUE);

    if (instance == NULL) {
        printf("hello_worldTypeSupport_create_data error\n");
        publisher_shutdown(participant);
        return -1;
    }

    /* For a data type that has a key, if the same instance is going to be
       written multiple times, initialize the key here
       and register the keyed instance prior to writing */
    /*
        instance_handle = hello_worldDataWriter_register_instance(
            hello_world_writer, instance);
    */

    /* Main loop */
    for (count = 0; (sample_count == 0) || (count < sample_count); ++count) {
        printf("Writing hello_world, count %d\n", initial_count);

        /* Modify the data to be written here */
        instance->data = initial_count;
        initial_count++;

        /* Write data */
        retcode = hello_worldDataWriter_write(
                hello_world_writer,
                instance,
                &instance_handle);
        if (retcode != DDS_RETCODE_OK) {
            printf("write error %d\n", retcode);
        }

        NDDS_Utility_sleep(&send_period);
    }

    while (sleep != 0) {
        NDDS_Utility_sleep(&one_sec);
        sleep--;
    }

    /*
        retcode = hello_worldDataWriter_unregister_instance(
            hello_world_writer, instance, &instance_handle);
        if (retcode != DDS_RETCODE_OK) {
            printf("unregister instance error %d\n", retcode);
        }
    */

    /* Delete data sample */
    retcode = hello_worldTypeSupport_delete_data_ex(instance, DDS_BOOLEAN_TRUE);
    if (retcode != DDS_RETCODE_OK) {
        printf("hello_worldTypeSupport_delete_data error %d\n", retcode);
    }

    /* Cleanup and delete delete all entities */
    return publisher_shutdown(participant);
}

int main(int argc, char *argv[])
{
    int domain_id = 0;
    int sample_count = 0; /* infinite loop */
    int initial_value = 0;
    int dwh = 0;
    int sleep = 0;
    char syntax[1024];
    int i;

    sprintf(syntax,
            "%s [options] \n\
    -domain_id <domain ID> (default: 0)\n\
    -sample_count <number of published samples> (default: infinite)\n\
    -initial_value <first sample value> (default: 0)\n\
    -sleep <sleep time in seconds before finishing> (default: 0)\n\
    -dwh <1|0> Enable/Disable durable writer history (default: 0)\n",
            argv[0]);

    for (i = 1; i < argc;) {
        READ_INTEGER_PARAM("-sleep", sleep);
        READ_INTEGER_PARAM("-domain_id", domain_id);
        READ_INTEGER_PARAM("-sample_count", sample_count);
        READ_INTEGER_PARAM("-initial_value", initial_value);
        READ_INTEGER_PARAM("-dwh", dwh);
        printf("%s", syntax);
        return 0;
    }

    /* Uncomment this to turn on additional logging
    NDDS_Config_Logger_set_verbosity_by_category(
        NDDS_Config_Logger_get_instance(),
        NDDS_CONFIG_LOG_CATEGORY_API,
        NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return publisher_main(domain_id, sample_count, initial_value, dwh, sleep);
}

#ifdef RTI_VX653
const unsigned char *__ctype = NULL;

void usrAppInit()
{
    #ifdef USER_APPL_INIT
    USER_APPL_INIT; /* for backwards compatibility */
    #endif

    /* add application specific code here */
    taskSpawn(
            "pub",
            RTI_OSAPI_THREAD_PRIORITY_NORMAL,
            0x8,
            0x150000,
            (FUNCPTR) publisher_main,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0);
}
#endif
